/* attr.c - backend routines for dealing with attributes */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "slap.h"
#include "back-ldbm.h"

extern char	**str2charray();

static int
ainfo_type_cmp(
    char		*type,
    struct attrinfo	*a
)
{
	return( strcasecmp( type, a->ai_type ) );
}

static int
ainfo_cmp(
    struct attrinfo	*a,
    struct attrinfo	*b
)
{
	return( strcasecmp( a->ai_type, b->ai_type ) );
}

void
attr_masks(
    struct ldbminfo	*li,
    char		*type,
    int			*indexmask,
    int			*syntaxmask
)
{
	struct attrinfo	*a;

	if ( (a = (struct attrinfo *) avl_find( li->li_attrs, type,
	    ainfo_type_cmp )) == NULL ) {
		if ( (a = (struct attrinfo *) avl_find( li->li_attrs, "default",
		    ainfo_type_cmp )) == NULL ) {
			a = (struct attrinfo *) ch_malloc( sizeof(struct
			    attrinfo) );
			a->ai_type = strdup( type );
			a->ai_syntaxmask = attr_syntax( a->ai_type );
			a->ai_indexmask = 0;

			if ( avl_insert( &li->li_attrs, a, ainfo_cmp,
			    avl_dup_error ) != 0 ) {
				Debug( LDAP_DEBUG_ANY,
				    "could not insert default attr info\n",
				    0, 0, 0 );
				free( a->ai_type );
				free( (char *) a );
			}
		}
	}
	if ( a->ai_syntaxmask == 0 ) {
		a->ai_syntaxmask = attr_syntax( type );
	}

	*indexmask = a->ai_indexmask;
	*syntaxmask = a->ai_syntaxmask;
}

void
attr_index_config(
    struct ldbminfo	*li,
    char		*fname,
    int			lineno,
    int			argc,
    char		**argv
)
{
	int		i, j;
	char		**attrs, **indexes;
	struct attrinfo	*a;

	attrs = str2charray( argv[0], "," );
	if ( argc > 1 ) {
		indexes = str2charray( argv[1], "," );
	}
	for ( i = 0; attrs[i] != NULL; i++ ) {
		a = (struct attrinfo *) ch_malloc( sizeof(struct attrinfo) );
		a->ai_type = strdup( attrs[i] );
		a->ai_syntaxmask = attr_syntax( a->ai_type );
		if ( argc == 1 ) {
			a->ai_indexmask = (INDEX_PRESENCE | INDEX_EQUALITY |
			    INDEX_APPROX | INDEX_SUB);
		} else {
			a->ai_indexmask = 0;
			for ( j = 0; indexes[j] != NULL; j++ ) {
				if ( strncasecmp( indexes[j], "pres", 4 )
				    == 0 ) {
					a->ai_indexmask |= INDEX_PRESENCE;
				} else if ( strncasecmp( indexes[j], "eq", 2 )
				    == 0 ) {
					a->ai_indexmask |= INDEX_EQUALITY;
				} else if ( strncasecmp( indexes[j], "approx",
				    6 ) == 0 ) {
					a->ai_indexmask |= INDEX_APPROX;
				} else if ( strncasecmp( indexes[j], "sub", 3 )
				    == 0 ) {
					a->ai_indexmask |= INDEX_SUB;
				} else if ( strncasecmp( indexes[j], "none", 4 )
				    == 0 ) {
					if ( a->ai_indexmask != 0 ) {
						fprintf( stderr,
"%s: line %d: index type \"none\" cannot be combined with other types\n",
						    fname, lineno );
					}
					a->ai_indexmask = 0;
				} else {
					fprintf( stderr,
			"%s: line %d: unknown index type \"%s\" (ignored)\n",
					    fname, lineno, indexes[j] );
					fprintf( stderr,
	"valid index types are \"pres\", \"eq\", \"approx\", or \"sub\"\n" );
				}
			}
		}

		if ( avl_insert( &li->li_attrs, a, ainfo_cmp, avl_dup_error )
		    != 0 ) {
			fprintf( stderr,
    "%s: line %d: duplicate index definition for attr \"%s\" (ignored)\n",
			    fname, lineno, a->ai_type );
			free( a->ai_type );
			free( (char *) a );
		}
	}
	charray_free( attrs );
	if ( argc > 1 )
		charray_free( indexes );
}
