/*
 * Copyright (c) 1990, 1995 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */


#include <stdio.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/signal.h>

#include "slurp.h"

extern	int	no_work_interval;
extern	int	is_fatal( int );

extern	int	slurpd_shutdown;
extern	int	one_shot_mode;

/* Forward references */
int file_nonempty( char * );
static void set_shutdown();




/*
 * If "replogfile" exists and is non-empty, call copy_replog() to make a
 * private copy, then truncate it.  Once the file has been copied, call
 * distribute_work() to parcel out the work to each replica.
 *
 * If the file is empty, sleep a while, then wake up and check again.
 */
void
service_replicas(
char *replogfile,
char *myreplogfile )
{
    int		rc;

    (void) signal( SIGHUP, set_shutdown );
    (void) signal( SIGUSR2, set_shutdown );
    (void) signal( SIGINT, set_shutdown );

    Debug( LDAP_DEBUG_ARGS,
	"Trace: service_replicas (%d): replogfile = %s, myreplogfile = %s\n",
	    getpid(), replogfile, myreplogfile );

    while ( !slurpd_shutdown ) {
	/*
	 * See if there's any work to be done
	 */
	if ( file_nonempty( replogfile )) {
	    Debug( LDAP_DEBUG_ARGS,
		"Trace: service_replicas (%d) processing: , myreplogfile = %s\n",
		    getpid(), myreplogfile, 0 );
	    if (( rc = copy_replog( replogfile, myreplogfile )) == 0 ) {
		distribute_work( myreplogfile );
	    } else {
		/* handle error */
		if ( rc < 0 ) {
		    Debug( LDAP_DEBUG_ANY,
			    "Error: Exiting - fatal error\n", 0, 0, 0 );
		    exit( -1 );
		}
	    }
	} else {
	    if ( one_shot_mode ) {
		Debug( LDAP_DEBUG_ARGS,
			"Trace: service_replicas (%d): one_shot_mode, exit\n",
			getpid(), 0, 0 );
		exit( 0 );
	    }
	    Debug( LDAP_DEBUG_ARGS,
		    "Trace: service_replicas (%d): file %s empty, sleeping\n", 
		    getpid(), replogfile, 0 );
	    sleep( no_work_interval );
	}
    }
    exit( 0 );
}




/*
 * Return 1 if the given file exists and has a nonzero size,
 * 0 if it is empty or nonexistent.
 */
int
file_nonempty( char *filename )
{
    static struct stat 	stbuf;

    if ( stat( filename, &stbuf ) < 0 ) {
	return( 0 );
    } else {
	Debug( LDAP_DEBUG_ARGS,
		"Trace: file_nonempty (%d): size of \"%s\"is %ld\n",
		getpid(), filename, stbuf.st_size );
	return( stbuf.st_size > (off_t ) 0 );
    }
}


/*
 * Signal handler for HUP signal.  Sets the flag which will cause
 * slurpd to clean up and terminate.
 */
static void
set_shutdown()
{
    Debug( LDAP_DEBUG_ANY,
	    "slurpd (pid %d) received signal - shutting down\n", getpid(), 0,
   	    0 );
    slurpd_shutdown = 1;
}
