/*
 * Copyright (c) 1990, 1995 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */
/* slurp.h - Standalone Ldap Update Replication Daemon (slurpd) */

#ifndef _SLURPD_H_
#define _SLURPD_H_

#define LDAP_SYSLOG

#include <syslog.h>
#include <errno.h>
#include <sys/types.h>
#include "lber.h"
#include "ldap.h"

#define	DEFAULT_SLURPD_REPLICA_DIR	"/usr/tmp"
#define	DEFAULT_SLURPD_REPLOGFILE	"slurpd.replog"
/* default srvtab file.  Can be overridden */
#define	SRVTAB				"/etc/srvtab"

/* Amount of time to sleep if no more work to do */
#define	DEFAULT_NO_WORK_INTERVAL	3

#define	NEED_TO_TRIM			1
#define	NO_MORE_WORK			2

/* Only try to trim slurpd replicas files larger than this size */
#define	MIN_TRIM_FILESIZE		( 10L * 1024L )

/* Maximum line length we can read from replication log */
#define	REPLBUFLEN			256

#define	AUTH_SIMPLE	1
#define	AUTH_KERBEROS	2

#define	ERROR_STR	"ERROR"

typedef struct replicainfo {
    char	*repl_hostname;		/* Canonical hostname of replica */
    int		repl_port;
    int		repl_bind_method;	/* AUTH_SIMPLE or AUTH_KERBEROS */
    char	*repl_bind_dn;		/* DN to bind as when replicating */
    char	*repl_password;		/* Password for AUTH_SIMPLE */
    pid_t	repl_pid;		/* replication process pid, 0 = none */
    char	*repl_prin;		/* principal to use for krb bind */
    LDAP	*repl_ldp;		/* LDAP struct for this replica */
    char	*repl_srvtab;		/* srvtab file for this replica */
    struct replicainfo	*repl_next;	
} replicainfo;

typedef struct repl_s {
    /* null-terminated list of replica hostnames/ports in "host:port" format */
    char	**replicas;
    int		nreplicas;
    char	*replbuf;	/* ptr to contents following replica: lines */
} repl_s;

typedef struct modinfo {
    char	*type;
    char	*value;
    int		len;
} modinfo;

typedef struct repl {
    char	**replicas;	/* null-terminated list of replica hostnames */
    int		nreplicas;
    char	*time;		/* time of modification */
    char	*dn;		/* dn of entry being modified */
    int		changetype;	/* type of change */
    modinfo	*mods;		/* modifications to make */
} repl;

typedef struct repl_instance {
    char			*ri_replicaname;	/* hostname of server */
    pid_t			ri_pid;			/* child pid */
    struct repl_instance	*ri_next;		/* next instance */
} repl_instance;

#define	T_CHANGETYPESTR		"changetype"
#define	T_CHANGETYPE		1
#define	T_TIMESTR		"time"
#define	T_TIME			2
#define	T_DNSTR			"dn"
#define	T_DN			3

#define	T_ADDCTSTR		"add"
#define	T_ADDCT			4
#define	T_MODIFYCTSTR		"modify"
#define	T_MODIFYCT		5
#define	T_DELETECTSTR		"delete"
#define	T_DELETECT		6
#define	T_MODRDNCTSTR		"modrdn"
#define	T_MODRDNCT		7

#define	T_MODOPADDSTR		"add"
#define	T_MODOPADD		8
#define	T_MODOPREPLACESTR	"replace"
#define	T_MODOPREPLACE		9
#define	T_MODOPDELETESTR	"delete"
#define	T_MODOPDELETE		10
#define	T_MODSEPSTR		"-"
#define	T_MODSEP		11

#define	T_NEWRDNSTR		"newrdn"

#define	T_ERR			-1

/* COnfig file keywords */
#define	HOSTSTR			"host"
#define	BINDDNSTR		"binddn"
#define	BINDMETHSTR		"bindmethod"
#define	KERBEROSSTR		"kerberos"
#define	SIMPLESTR		"simple"
#define	CREDSTR			"credentials"
#define BINDPSTR		"bindprincipal"
#define	SRVTABSTR		"srvtab"

#define	MAXREPLICAS		100

#define	REPLICA_SLEEP_TIME	( 10 )

#define BIND_OK 			0
#define BIND_ERR_BADLDP			1
#define	BIND_ERR_OPEN			2
#define	BIND_ERR_BAD_ATYPE		3
#define	BIND_ERR_SIMPLE_FAILED		4
#define	BIND_ERR_KERBEROS_FAILED	5

#endif /* _SLURPD_H_ */
